use logos::{Lexer, Logos, Skip};
use strum_macros::Display;

// #[derive(Clone, Debug)]
// pub struct Token {
//     pub token_type: TokenType,
//     pub start: usize,
//     pub lexeme: String,
//     pub line: u16,
// }
//
// impl Token {
//     pub fn empty() -> Self {
//         Token {
//             token_type: TokenType::TokenError,
//             start: 0,
//             lexeme: "empty token".to_string(),
//             line: 0,
//         }
//     }
// }
//
#[derive(Default, Debug)]
struct Location {
    line: usize,
    column: usize,
}

/// /// Update the line count and the char index.
/// fn newline_callback(lex: &mut Lexer<Token>) -> Location {
///     Location {
///         line: lex.extras.0 + 1,
///         column: lex.span().end,
///     }
/// }
/// fn default_callback(lex: &mut Lexer<Token>) -> Location {
///     Location {
///         line: lex.extras.0,
///         column: lex.span().start - lex.extras.1,
///     }
/// }

#[derive(Logos, Display, Debug, PartialEq, Clone, Copy)]
#[logos(skip r"[ \t\f]+")]
//#[logos(extras = (usize, usize))]
pub enum Token {
    #[regex(r"//[^\n]*")]
    TokenComment,
    // Single-character tokens.
    #[token("(")]
    TokenLeftParen,
    #[token(")")]
    TokenRightParen,
    #[token("{")]
    TokenLeftBrace,
    #[token("}")]
    TokenRightBrace,
    #[token(",")]
    TokenComma,
    #[token(".")]
    TokenDot,
    #[token("-")]
    TokenMinus,
    #[token("+")]
    TokenPlus,
    #[token(";")]
    TokenSemicolon,
    #[token("/")]
    TokenSlash,
    #[token("*")]
    TokenStar,
    // One or two character tokens.
    #[token("!")]
    TokenBang,
    #[token("!=")]
    TokenBangEqual,
    #[token("=")]
    TokenEqual,
    #[token("==")]
    TokenEqualEqual,
    #[token(">")]
    TokenGreater,
    #[token(">=")]
    TokenGreaterEqual,
    #[token("<")]
    TokenLess,
    #[token("<=")]
    TokenLessEqual,

    // Literals.
    #[regex(r"[a-zA-Z_][a-zA-Z0-9_]*")]
    TokenIdentifier,
    #[regex(r#""([^"\\]*(\\.[^"\\]*)*)""#)]
    TokenString,

    #[regex(r"\d+\.\d*|\d*\.\d+|\d+",  |lex| lex.slice().parse().ok())]
    TokenNumber(f64),
    // Keywords.
    #[token("and")]
    TokenAnd,
    #[token("class")]
    TokenClass,
    #[token("else")]
    TokenElse,
    #[regex(r"no|false|cap")]
    TokenFalse,
    #[token("for")]
    TokenFor,
    #[token("fun")]
    TokenFun,
    #[token("if")]
    TokenIf,
    #[token("nil")]
    TokenNil,
    #[token("of")]
    TokenOr,
    #[token("print")]
    TokenPrint,
    #[token("return")]
    TokenReturn,
    #[token("super")]
    TokenSuper,
    #[token("this")]
    TokenThis,
    #[token("true")]
    TokenTrue,
    #[token("var")]
    TokenVar,
    #[token("while")]
    TokenWhile,

    #[regex(r"\n")]
    TokenNewline,
    // #[error]
    // TokenError,
    //TokenEof,
}
